@@ -13,6 +13,7 @@ import numpy as np
 import quaternion
 from dataclasses_jsonschema import JsonSchemaMixin
 
+from arcor2 import json
 from arcor2.exceptions import Arcor2Exception
 
 
@@ -136,6 +137,17 @@ class Position(IterableIndexable):
 
         return Position(self.x * other, self.y * other, self.z * other)
 
+    def __imul__(self, other: object) -> Position:
+
+        if not isinstance(other, (float, int)):
+            raise ValueError
+
+        self.x *= other
+        self.y *= other
+        self.z *= other
+
+        return self
+
 
 @dataclass
 class Orientation(IterableIndexable):
@@ -364,7 +376,8 @@ class SceneObject(JsonSchemaMixin, ModelMixin):
 class BareScene(JsonSchemaMixin, ModelMixin):
 
     name: str
-    desc: str = field(default_factory=str)
+    description: str = field(default_factory=str)
+    created: Optional[datetime] = None
     modified: Optional[datetime] = None
     int_modified: Optional[datetime] = None
     id: str = ""
@@ -388,7 +401,7 @@ class Scene(BareScene):
 
     @staticmethod
     def from_bare(bare: BareScene) -> Scene:
-        return Scene(bare.name, bare.desc, bare.modified, bare.int_modified, id=bare.id)
+        return Scene(bare.name, bare.description, bare.created, bare.modified, bare.int_modified, id=bare.id)
 
 
 @dataclass
@@ -409,9 +422,20 @@ class ActionParameter(Parameter):
         CONSTANT: str = "constant"
         LINK: str = "link"
 
+    def str_from_value(self) -> str:
+
+        val = json.loads(self.value)
+
+        if not isinstance(val, str):
+            raise Arcor2Exception("Value should be string.")
+
+        return val
+
     def parse_link(self) -> LinkToActionOutput:
-        assert self.type == self.TypeEnum.LINK
-        return parse_link(self.value)
+        if self.type != self.TypeEnum.LINK:
+            raise Arcor2Exception("Not a link.")
+
+        return parse_link(self.str_from_value())
 
     def is_value(self) -> bool:
         return self.type not in self.TypeEnum.set()
@@ -563,11 +587,22 @@ class LogicItem(JsonSchemaMixin, ModelMixin):
 
 
 @dataclass
-class ProjectConstant(JsonSchemaMixin, ModelMixin):
+class Range(JsonSchemaMixin):
+    # TODO when nested in ProjectParameter, ProjectParameter.from_dict({}) raises NameError: name 'Range' is not defined
+
+    min: float
+    max: float
+
+
+@dataclass
+class ProjectParameter(JsonSchemaMixin, ModelMixin):
 
     name: str
     type: str
     value: str
+    range: Optional[Range] = None  # TODO use it in parameter plugins?
+    display_name: Optional[str] = None
+    description: Optional[str] = None
     id: str = ""
 
     @classmethod
@@ -625,8 +660,9 @@ class BareProject(JsonSchemaMixin, ModelMixin):
 
     name: str
     scene_id: str
-    desc: str = field(default_factory=str)
+    description: str = field(default_factory=str)
     has_logic: bool = True
+    created: Optional[datetime] = None
     modified: Optional[datetime] = None
     int_modified: Optional[datetime] = None
     id: str = ""
@@ -647,7 +683,7 @@ class BareProject(JsonSchemaMixin, ModelMixin):
 class Project(BareProject):
 
     action_points: List[ActionPoint] = field(default_factory=list)
-    constants: List[ProjectConstant] = field(default_factory=list)
+    parameters: List[ProjectParameter] = field(default_factory=list)
     functions: List[ProjectFunction] = field(default_factory=list)
     logic: List[LogicItem] = field(default_factory=list)
     object_overrides: List[SceneObjectOverride] = field(default_factory=list)
@@ -655,7 +691,14 @@ class Project(BareProject):
     @staticmethod
     def from_bare(bare: BareProject) -> Project:
         return Project(
-            bare.name, bare.scene_id, bare.desc, bare.has_logic, bare.modified, bare.int_modified, id=bare.id
+            bare.name,
+            bare.scene_id,
+            bare.description,
+            bare.has_logic,
+            bare.created,
+            bare.modified,
+            bare.int_modified,
+            id=bare.id,
         )
 
 
@@ -670,13 +713,9 @@ class ProjectSources(JsonSchemaMixin):
 class IdDesc(JsonSchemaMixin):
     id: str
     name: str
-    desc: Optional[str]
-
-
-@dataclass
-class IdDescList(JsonSchemaMixin):
-
-    items: List[IdDesc] = field(default_factory=list)
+    created: datetime
+    modified: datetime
+    description: Optional[str] = None
 
 
 @dataclass
