@@ -1,6 +1,5 @@
 import json
 import logging
-import os
 from typing import List, Optional, Tuple, Type, Union
 
 from apispec import APISpec
@@ -11,9 +10,10 @@ from flask import Flask, Response, jsonify
 from flask_cors import CORS
 from flask_swagger_ui import get_swaggerui_blueprint
 
+from arcor2 import env
 from arcor2.exceptions import Arcor2Exception
 
-RespT = Union[Response, Tuple[str, int]]
+RespT = Union[Response, Tuple[str, int], Tuple[Response, int]]
 
 
 class FlaskException(Arcor2Exception):
@@ -60,14 +60,14 @@ def run_app(
         return
 
     @app.route("/swagger/api/swagger.json", methods=["GET"])
-    def get_swagger() -> str:
+    def get_swagger() -> RespT:
         return jsonify(spec.to_dict())
 
-    @app.errorhandler(Arcor2Exception)
+    @app.errorhandler(Arcor2Exception)  # type: ignore  # TODO what's wrong?
     def handle_bad_request_general(e: Arcor2Exception) -> Tuple[str, int]:
         return json.dumps(str(e)), 400
 
-    @app.errorhandler(FlaskException)
+    @app.errorhandler(FlaskException)  # type: ignore  # TODO what's wrong?
     def handle_bad_request_intentional(e: FlaskException) -> Tuple[str, int]:
         return json.dumps(str(e)), e.error_code
 
@@ -80,7 +80,7 @@ def run_app(
     # Register blueprint at URL
     app.register_blueprint(swaggerui_blueprint, url_prefix=SWAGGER_URL)
 
-    if not os.getenv("ARCOR2_REST_API_DEBUG", False):
+    if not env.get_bool("ARCOR2_REST_API_DEBUG", False):
         # turn off logging each endpoint call by default
         log = logging.getLogger("werkzeug")
         log.setLevel(logging.ERROR)
