@@ -1,7 +1,6 @@
+from ast import Str
 from typing import Any, List
 
-from typed_ast.ast3 import Str
-
 from arcor2.cached import CachedProject as CProject
 from arcor2.cached import CachedScene as CScene
 from arcor2.parameter_plugins.base import ParameterPlugin, TypesDict
